%MATLAB script to generate longevity statistics and cohort projection for the
%the matrix formulation of the gamma-Gompertz model
%
% Supplement to:
% Caswell, H. 2014. A matrix approach to the statistics of longevity in the
% gamma-Gompertz and related heterogeneous frailty models. Demographic Research
%
% requires function spvecperm.m to compute sparse vec-permutation matrix


%G-G parameters for Sweden 1950, from Missov (2013, Table 1)
a=0.0340;
b=0.12;
k=8.23;
a=a*exp(-70*b);
lambda=1/k;

%parameters for the matrix model:
%number of frailty groups
g=100;
%number of age classes
s=150;

%create array z of log-spaced frailty values
%minimum
xmin=1.0e-7;
% maximum value, where cdf reaches 0.9999
xmax=gaminv(0.9999,k,lambda);

%column vector of frailty intervals log spaced

x=logspace(log10(xmin), log10(xmax),g+1)';

%width of frailty intervals
deltax=diff(x);

%vector of frailty values, at midpoints of intervals
z(:,1)=0.5*(x(1:end-1)+x(2:end));

%initial distribution of frailty
pi=gampdf(z,k,lambda).*deltax;
%make sure pi sums to 1
pi=pi/sum(pi);

%baseline mortality schedule (Gompertz)
%vector of ages
y=0:(s-1);
%Gompertz mortality rate vector
mu=a*exp(b*y);
%make mu a column vector
mu=mu';

% create sxg array of frailty-specific mortality schedules
muarray=mu*z';

%some useful matrices needed for the model

Is=speye(s);
Ig=speye(g);
es=ones(s,1);
eg=ones(g,1);
%sparse vec permutation matrix
K=spvecperm(s,g);

%create survival matrices for each frailty class
for i=1:g
    p=exp(-muarray(:,i));
    U{i}=spdiags(p,-1,s,s);
end

%create frailty transition matrices for each age class
for i=1:s
    M{i}=speye(g);
end

%create block survival matrix
bbU=kron(Ig(:,1)*Ig(1,:),U{1});
for i=2:g
    bbU=bbU+kron(Ig(:,i)*Ig(i,:),U{i});
end

%create block frailty transition matrix
bbM=kron(Is(:,1)*Is(1,:),M{1});
for i=2:s
    bbM=bbM+kron(Is(:,i)*Is(i,:),M{i});
end

% note: for the special case of the G-G model
% M=speye(g);
% bbM=kron(speye(s),M);

%create cohort projection matrix, Utilde
Utilde=bbM*K*bbU*K';

%generate fundamental matrix
Ntilde=(speye(s*g)-Utilde)\speye(s*g);

%compute marginal survivorship function
lxstar=kron(Is,ones(g,1)')*Ntilde*kron(Is(:,1),pi);
%compute marginal mortality rate function
mustar=-diff(log(lxstar));

%compute marginal fundamental matrix
for i=1:s
    Nstar(:,i)=lxstar/lxstar(i);
    Nstar(1:i-1,i)=0;
end

%now calculate the higher moments
%higher  moments of sojurn times
N1=Nstar;
N2=(2*(Is.*N1)-Is)*N1;
N3=(6*((Is.*N1)^2) - 6*(Is.*N1) + Is)*N1;

%moments of marginal longevity
eta1=(es'*N1)';
eta2=(eta1'*(2*N1 -Is))';
eta3=(eta1'*(6*N1^2 - 6*N1 +Is))';

%variance of longevity
vareta=eta2 - eta1.^2;

%following may be required because of rounding error
vareta(end)=0;
%standard deviation of longevity
sdeta=sqrt(vareta);
%coefficient of variation of longevity
cveta=sdeta./eta1;

%eliminate zero standard deviations (they appear in the last age class)

pick=find(sdeta==0);
sd2=sdeta;
sd2(pick)=1;

%skewness of longevity
skeweta=(diag(1./sd2)^3)*(2*(eta1.*eta1.*eta1) ...
    -3*(eta1.*eta2) + eta3);

%simulation

%create initial cohort, with distribution pi of frailty in first age class
n=zeros(s*g,1);
n(1:g)=pi;
p=n/sum(n);

%initialize output arrays
nstar=sparse(s,s);
pstar=sparse(s,s);

mstar=sparse(g,s);
pistar=sparse(g,s);

for i=1:s
    nout(:,i)=n;
    pout(:,i)=p;
    
    n=Utilde*n;
    p=Utilde*p/sum(Utilde*p);
    
end

nstar=kron(Is,eg')*nout;
pstar=kron(Is,eg')*pout;

mstar=kron(es',Ig)*nout;
pistar=kron(es',Ig)*pout;

%mean of frailty distribution
meanz=z'*pistar;

%standard deviation of fraility distribution
varz=(z.*z)'*pistar - meanz.^2;
stdz=sqrt(varz);



